@@ -1,6 +1,6 @@
 '''
 Luke-zhang-04
-Canvas Plus v1.1.2 (https://github.com/Luke-zhang-04/CanvasPlus)
+Canvas Plus v1.1.3 (https://github.com/Luke-zhang-04/CanvasPlus)
 Copyright (C) 2020 Luke-zhang-04
 
 This program is free software: you can redistribute it and/or modify
@@ -10,11 +10,11 @@ the Free Software Foundation, either version 3 of the License, or
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see https://github.com/Luke-zhang-04/CanvasPlus/blob/master/LICENSE.
+along with this program. If not, see https://github.com/Luke-zhang-04/CanvasPlus/blob/master/LICENSE.
 '''
 
 #tkinter
@@ -30,11 +30,13 @@ import cmath, math
 from numbers import Real
 
 #typing
-from typing import Tuple
+from typing import Tuple, Union, List, Callable, Dict
 
 #warnings
 import warnings
 
+print("Hello from CanvasPlus")
+
 class Error(Exception):
    '''Base class for other exceptions'''
    pass
@@ -51,6 +53,7 @@ class InvalidObjectType(Error):
     '''raised when object type not supported'''
     pass
 
+
 class WidgetWindows:
     '''Class for createing widgets as windows within the canvas'''
 
@@ -167,9 +170,56 @@ class WidgetWindows:
         return self._create_widget(x, y, Spinbox, **kwargs)
 
 
-class CanvasPlus(Canvas, WidgetWindows):
+class Transformations:
+    def rotate(self, tagOrId: Union[int, str], x: Real, y: Real, amount: Real, unit: str = "rad", warn: bool = True) -> None:
+        '''rotate obj on axis x, y by amount in degrees or radians clockwise'''
+        if unit in ("d", "deg", "degree", "degrees"):
+            amount *= math.pi/180 #convert to radians
+        elif unit in ("r", "rad", "radian", "radians"):
+            pass
+        else:
+            raise InvalidUnitError("Invalid unit \"" + unit + "\"")
+        
+        angle = cmath.exp(amount*1j)
+        offset = complex(x, y)
+        newCords = []
+        cords = [
+            (self.coords(tagOrId)[i], self.coords(tagOrId)[i+1]) for i in range(0, len(self.coords(tagOrId)), 2)
+        ]
+        for xPt, yPt in cords:
+            num = angle * (complex(xPt, yPt) - offset) + offset
+            newCords.append(num.real)
+            newCords.append(num.imag)
+        
+        objType = self.tk.call(self._w, 'type', tagOrId)
+        if objType == "polygon":
+            self.coords(tagOrId, *newCords)
+        else:
+            if (warn):
+                warnings.warn(
+                    "WARNING! Canvas element of type " + objType + " is not supported. Rotation may not look as expected. " + 
+                    "Use the to_polygon() method to turn the " + objType + " into a polygon.",
+                    UnsupportedObjectType
+                )
+            self.coords(tagOrId, *newCords)
+
+
+class CanvasPlus(Canvas, WidgetWindowsm, Transformations):
     '''Improved Canvas widget with more functionality to display graphical elements like lines or text.'''
 
+    def clone(self, tagOrId: Union[int, str], *args: List[int]) -> int:
+        '''clones tagOrId and places is at optional coordinates, or places is on top of the first object'''
+        if len(args) == 0:
+            args = self.coords(tagOrId)
+        
+        output = self.get_attributes(tagOrId)
+        
+        return self._create(
+            self.tk.call(self._w, 'type', tagOrId),
+            args,
+            output
+        )
+        
     def create_arrow(self, x1: Real, y1: Real, headLength: Real, headWidth: Real, bodyLength: Real, bodyWidth: Real, **kwargs) -> int:
         '''Create arrow with x1, y1 as the tip; headWith, headLengh as the length and width of the arrowhead; and bodyLength, bodyWidth as the length and width of the arrow body, as well as direction = val (0 by default)'''
         
@@ -189,61 +239,6 @@ class CanvasPlus(Canvas, WidgetWindows):
         '''Create circle with coordinates x, y, radius'''
         return self._create('oval', [x+radius, y+radius, x-radius, y-radius], kwargs)
 
-    def to_polygon(self, obj:int) -> int:
-        '''converts rectangle to polygon'''
-        properties = self.itemconfig(obj)
-        output = {
-            key: properties[key][-1] for key in properties
-        }
-        
-        cords = [self.tk.getdouble(x) for x in self.tk.splitlist(self.tk.call((self._w, 'coords') + tuple([obj])))]
-
-        if output["width"] == '0.0':
-            output["outline"] = ''
-
-        if self.tk.call(self._w, 'type', obj) == "rectangle":
-            newCords = [
-                cords[0], cords[1],
-                cords[1], cords[2],
-                cords[2], cords[3],
-                cords[3], cords[0]
-            ]
-        else:
-            raise InvalidObjectType("Invalid canvas element \"" + self.tk.call(self._w, 'type', obj) + "\"")
-
-        self.tk.call((self._w, 'delete') + tuple([obj]))
-
-        return self._create('polygon', newCords, output)
-
-    poly = to_polygon
-
-    def rotate(self, obj: int, x: Real, y: Real, amount: Real, unit: str = "rad") -> None:
-        '''rotate obj on axis x, y by amount in degrees or radians clockwise'''
-        if unit in ("d", "deg", "degree", "degrees"):
-            amount *= math.pi/180 #convert to radians
-        elif unit in ("r", "rad", "radian", "radians"):
-            pass
-        else:
-            raise InvalidUnitError("Invalid unit \"" + unit + "\"")
-        
-        angle = cmath.exp(amount*1j)
-        offset = complex(x, y)
-        newCords = []
-        cords = [
-            (self.coords(obj)[i], self.coords(obj)[i+1]) for i in range(0, len(self.coords(obj)), 2)
-        ]
-        for xPt, yPt in cords:
-            num = angle * (complex(xPt, yPt) - offset) + offset
-            newCords.append(num.real)
-            newCords.append(num.imag)
-        
-        objType = self.tk.call(self._w, 'type', obj)
-        if objType == "polygon":
-            self.coords(obj, *newCords)
-        else:
-            warnings.warn("WARNING! Canvas element of type " + objType + " is not supported. Rotation may not look as expected.", UnsupportedObjectType)
-            self.coords(obj, *newCords)
-
     def create_round_rectangle(self, x1: Real, y1: Real, x2: Real, y2: Real, radius: Real = 25, **kwargs) -> int:
         '''Create circle with coordinates x1, y1, x2, y2, radius = val (default 25)'''
         points = [
@@ -272,6 +267,70 @@ class CanvasPlus(Canvas, WidgetWindows):
         kwargs["smooth"] = True
         return self._create('polygon', points, kwargs)
 
+    def get_attributes(self, tagOrId: Union[int, str]) -> Dict:
+        '''Returns all properties of tagOrId'''
+        properties = self.itemconfig(tagOrId)
+        return {key: properties[key][-1] for key in properties}
+
+    get_attr = get_attributes
+
+    def __iter__(self) -> iter:
+        '''Creates iterator of everything on the canvas'''
+        return iter(self.find_all())
+
+    def to_polygon(self, tagOrId: Union[int, str]) -> int:
+        '''converts rectangle to polygon'''
+        output = self.get_attributes(tagOrId)
+
+        cords = [self.tk.getdouble(x) for x in self.tk.splitlist(self.tk.call((self._w, 'coords') + tuple([tagOrId])))]
+
+        if output["width"] == '0.0':
+            output["outline"] = ''
+
+        if self.tk.call(self._w, 'type', tagOrId) == "rectangle":
+            newCords = [
+                cords[0], cords[1],
+                cords[2], cords[1],
+                cords[2], cords[3],
+                cords[0], cords[3]
+            ]
+        else:
+            raise InvalidObjectType("Invalid canvas element \"" + self.tk.call(self._w, 'type', tagOrId) + "\"")
+
+        self.tk.call((self._w, 'delete') + tuple([tagOrId]))
+
+        return self._create('polygon', newCords, output)
+
+    poly = to_polygon
+
+    def tags_bind(
+        self, tagsOrIds: Union[Union[int, str], Tuple[Union[int, str]], List[Union[int, str]]],
+        sequences: Union[str, Tuple[str], List[str]] = None,
+        funcs = Union[Callable, Tuple[Callable], List[Callable]], add = None) -> Union[str, List[str]]:
+        '''Binds either multiple tags to one function, or multiple tags to multiple functions with matching indicies in one function
+        
+        i.e (tag1, tag2, tag3), func1 will bind tag1, tag2, tag3 into fun1, while (tag1, tag2, tag3), (func1, func2, func3) will bind tag1 to func1, tag2 to func2, tag3 to func3.
+        '''
+        if type(tagsOrIds) == int and callable(funcs): #normal tag_bind
+            return self._bind((self._w, 'bind', tagsOrIds),sequences, funcs, add)
+        else:
+            bindings = []
+            if type(funcs) not in [list, tuple]:
+                funcs = tuple([funcs])
+            if type(sequences) not in [list, tuple]:
+                sequences = tuple([sequences])
+
+            for index, obj in enumerate(tagsOrIds):
+                bindings.append(
+                        self._bind((self._w, 'bind', obj),
+                        sequences[index%len(sequences)],
+                        funcs[index%len(funcs)],
+                        add
+                    )
+                )
+            return bindings
+
+
 def _test():
     #Imports
     from tkinter import Tk, StringVar, DoubleVar
@@ -283,11 +342,11 @@ def _test():
     canvas.pack()
 
     #create circle function
-    canvas.create_circle(300, 300, 100, fill = "black", outline = "green", width = 3)
+    canvas.create_circle(300, 600, 100, fill = "black", outline = "green", width = 3)
     
     #create rounded rectangle function
     canvas.create_round_rectangle(
-        400, 400, 500, 500, radius = 75, fill = "blue", outline = "orange", width = 5
+        400, 550, 500, 650, radius = 75, fill = "blue", outline = "orange", width = 5
     )   
 
     #create arrow function and rotate it to by 310 degrees clockwise
@@ -295,14 +354,15 @@ def _test():
     canvas.rotate(arrow, 600, 600, 310, unit="deg")
 
     #create a rectangle and convert it to a polygon; then rotate it by pi/4 radians (45 degrees)
-    rect = canvas.create_rectangle(100, 100, 200, 200, fill = "#f7a8c6", width = 0)
+    rect = canvas.create_rectangle(100, 550, 200, 650, fill = "#f7a8c6", width = 0)
+    canvas.clone(rect)
     rect = canvas.poly(rect)
-    canvas.rotate(rect, 150, 150, math.pi/4)
+    canvas.rotate(rect, 150, 600, math.pi/4)
 
     #create an entry and set it's default value
     content = StringVar()
     canvas.create_entry(0, 0, anchor = "nw", textvariable = content, fg = "blue", bg = "gold")
-    content.set("a default value")
+    content.set("This is CanvasPlus v1.1.3")
 
     #create button to print the value in the previously cretaed entry
     canvas.create_button(
@@ -318,16 +378,13 @@ def _test():
 
     #create a label
     canvas.create_label(
-        5, 75, font = ("Times", "24"), fg = "black", bg = "green", text = "Hello World!", anchor = "nw"
-    )
-
-    #create a scale
-    canvas.create_scale(
-        5, 100, anchor = "nw", bg = "yellow", activebackground = "gold", from_ = 0, to = 100
+        5, 75, font = ("Times", "24"), fg = "black", bg = "green", text = "By Luke-zhang-04", anchor = "nw"
     )
 
     canvas.update()
     canvas.mainloop()
 
 if __name__ == "__main__":
-    _test()
\ No newline at end of file
+    _test()
+
+print("Imported successfully")
\ No newline at end of file
